/*
WeatherLAN: Sensor de temperatura y humedad con arduino
http://weatherlan.hfiel.es/

 Copyright 2011 Hector Fiel


 This file is part of WeatherLAN.

    WeatherLAN is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    WeatherLAN is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with WeatherLAN.  If not, see <http:www.gnu.org/licenses/>.


Autor: Héctor Fiel (http://hfiel.es)
Fichero: weatherlan_web.pde
Versión: 1
Revisión: Versión final.
Enlace: http://weatherlan.hfiel.es
Importante: Código probado en Arduino Duemilanove, en la versión 0018 del IDE.
Seguir las instrucciones para la instalacion de las librerias UDP necesarias (indicadas mas abajo, en la carga de la libreria).

Resumen version: version para 2 sensores de temperatura DS18B20, y un sensor de humedad 808H5V5.
Referencia de 5V, salida por serie y web, almacenamiento en BBDD remota
Sincronizacion de tiempo con servidor NTP
Busqueda automatica direccion sensores 1-Wire


Reconocimientos:
El codigo en general se ha desarrollado mediante los ejemplos de la web de arduino.cc
 http://arduino.cc/en/Tutorial/HomePage
El codigo de servidor y cliente web se basa en los ejemplos de las librerias ethernet de arduino.cc
 http://arduino.cc/en/Reference/Ethernet
El codigo para NTP se basa en el ejemplo TimeNTP.PDE de la libreria time de arduino.cc
 http://www.arduino.cc/playground/Code/Time
El código para los sensores de temperatura se basa en los ejemplos de arduino.cc y 
la libreria de temperatura 1-Wire de Miles Burton y el ejemplo multiple.pde
 http://www.arduino.cc/playground/Learning/OneWire
 http://milesburton.com/index.php/Dallas_Temperature_Control_Library

Descripcion:

La medida de temperatura se realiza mediante sensores digitales de Dallas/Maxim que funcionan con el
protocolo 1-Wire (permiten el envio y recepcion de datos usando una sola linea digital, con hasta
64 dispositivos compartiendo dicha linea). Se usa la configuración 1-Wire Normal (fuente de
alimentacion externa, total de 3 hilos) para agilizar la lectura de datos de los sensores. Se puede
usar el modo de alimentación parásito (dos hilos) pero esto implica un retardo de 750ms en cada lectura.
Los sensores se conectan a dos conectores en la placa, el orden de los mismos viene definido por
la búsqueda del bus que efectua la libreria, y es determinista (el sensor identificado como 0 siempre
sera el mismo, si no se cambian los sensores).

La medida de humedad se realiza mediante un sensor analógico conectado al DAC del Arduino.

Se realiza almacenamiento en BBDD, mediante una peticion web por GET al servidor remoto, donde se
ejecuta un servidor Web Apache con PHP y una BBDD MySQL
La peticion es de la forma:

http:\\<IP servidor>\<carpeta weatherlan>\sensor.php?reg=1&t1=14.1&t2=14.2&h1=50.5&hora=12&minuto=25&dia=28&mes=5&anyo=2010

donde los datos en cada campo get se insertan mediante las lecturas de los sensores.
La IP del servidor se indica en las variables de inicializacion de ethernet.
La carpeta de instalacion de weatherlan se indica en la funcion de conexion "envioweb".
Los parametros enviados por get son:
t1: temperatura sonda 1
t2: temperatura sonda 2
h1: humedad sonda 1
hora: hora de la medicion
minuto: minuto de la medicion
dia: dia de la medicion
mes: mes de la medicion
anyo: anyo de la medicion
Si un campo de sensor (t1, t2, h1) no se incluye, la bbdd almacena NULL en ese campo.

INFORMACION SOBRE PROTOCOLO DE TIEMPO NTP:
El protocolo NTP permite sincronizar la hora con un servidor de internet. Esto se usa para poder hacer
registros en el servidor remoto en tiempos exactos (PE, cada 15 minutos). El envio de la fecha y hora
desde el arduino permite garantizar que los datos se almacenan con la fecha exacta, sin depender de la
hora del sistema del servidor con PHP y la BBDD. Es importante tener en cuenta que el sistema completo
trabaja con hora UTC (universal), y no con la hora española. Esto permite evitar problemas con los
cambios de horario de verano (como tener dos registros con la misma hora el dia que se retrasa la hora).
Esto hay que tenerlo en cuenta en el servidor web a la hora de procesar los datos para mostrarlos.
Para el correcto funcionamiento, el servidor MyQSL de registro de datos debe tener configuradas correctamente las TimeZones

Los tres servicios de red (cliente web, servidor web y cliente ntp) se implementan mediante funciones separadas, para
facilitar el código principal.

//****
PINES ARDUINO EMPLEADOS:
//****
Pines usandos en las entradas analogicas de Arduino:
0: 808H5V5

Pines usados en las entradas digitales de Arduino:
3: 1 Wire (2 sensores DS18B20)

*/

//****
//CARGA DE LIBRERIAS
//****
//Se indica la libreria y la direccion de descarga
#include <Ethernet.h>  //Libreria Modulo Ethernet arduino (incluida en el IDE)

#include <Time.h>      //Libreria de tiempo arduino
//http://www.arduino.cc/playground/Code/Time

#include <UdpBytewise.h> //Libreria UDP de bjoern@cs.stanford.edu, para mensajes UDP
//http://bitbucket.org/bjoern/arduino_osc/src/14667490521f/libraries/Ethernet/
//NOTA: las librerias UDP tienen que estar en el directorio libraries\Ethernet del software IDE Arduino,
//no funcionan en el directorio de librerias del usuario
//hay que incluuir 4 ficheros:
//UdpRaw.h
//UdpRaw.cpp
//UdpBytewise.h
//UdpBytewise.cpp

//Librerias onewire
#include <OneWire.h> //Libreria para protocolo 1-Wire 
//(incluida en la descarga de DallasTemperature.h, mas abajo)
//http://www.pjrc.com/teensy/td_libs_OneWire.html

#include <DallasTemperature.h> //libreria para los sensores de temperatura Dallas/Maxim
//http://milesburton.com/index.php/Dallas_Temperature_Control_Library


//****
//DEFINICION DE PINES
//****
//Pin analógico a usar en el sensor de humedad
#define puertoH1 0

//Pin digital para los sensores de temperatura
#define ONE_WIRE_BUS 2


//****
//DECLARACION DE VARIABLES PARA CONEXIONES ETHERNET
//****


//****
//DIRECCION MAC
//****
//la direccion mac debe ser unica para cada dispositivo de la red
//esta viene puesta en el ejemplo de Arduino, aquí se ha cambiado el ultimo
//valor para evitar problemas con otros ejemplos (era 0xED, se pone 0xCD)
 byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xCD };
 
//****
//IP SENSOR WEATHERLAN
//****
//la direccion ip tambien tiene que ser única en la red, ponemos una que esté libre en 
//nuestro rango de direcciones (dependerá de la configuración de red de cada uno)
 byte ip[] = { 10, 66, 3, 101 };
 
//****
//GATEWAY (PUERTA DE ENLACE)
//****
//puerta de enlace (dependerá de la configuración de red de cada uno)
 byte gw[] = {10, 66, 0, 1};
 
//****
//MASCARA DE SUBRED
//****
//subred (dependerá de la configuración de red de cada uno)
 byte subnet[] = {255, 255, 255, 0}; 
 
//****
//IP SERVIDOR BBDD
//****
//definimos la ip del servidor remoto donde se va a conectar arduino para
//almacenar los datos, ajustar segun la IP del servidor de cada uno
 byte server[] = {10, 66, 3, 100};


//****
//IP SERVIDOR NTP
//****
//IP del servidor NTP, ajustar al servidor que quiera usar cada uno
//Por defecto, se usa la misma IP que el servidor de registros BBDD (habra que activar el servicio NTP)
byte SNTP_server_IP[] = {10, 66, 3, 100};
//en caso de no usar servidor NTP local, prueba con alguno de estos
//byte SNTP_server_IP[] = {150, 214, 94, 5};   //hora.roa.es: Real Observatorio de la Armada, hora ofical en españa
//byte SNTP_server_IP[] = { 192, 43, 244, 18}; // time.nist.gov
//byte SNTP_server_IP[] = { 130,149,17,21};    // ntps1-0.cs.tu-berlin.de
//byte SNTP_server_IP[] = { 192,53,103,108};   // ptbtime1.ptb.de

//****
//PUERTO DE ESCUCHA PARA EL SERVIDOR WEB LOCAL DEL SENSOR
//****
//puerto donde escucha peticiones web arduino, por defecto el 80 (HTTP)
Server servidor = Server(80);


//****
//DECLARACION DE VARIABLES PARA ALMACENAMIENTO DE DATOS
//****

//Variables donde almacenar las lecturas, con inicializacion a 0
//En el caso de la humedad, se usa un array para poder hacer
//multiples lecturas y obtener una media, dado que son datos
//analogicos y pueden oscilar bastante entre dos medidas.
//En las temperaturas, el sensor entrega un dato final y estable
//por lo que no necesitamos hacer medias
//Arrays donde almacenar datos crudos de humedad
 int vhumedad[5] = {0,0,0,0,0};
//variables para almacenar las medias
 float mhumedad = 0.0;
//variables con datos finales
 float temperaturaT1 = 0.0;
 float temperaturaT2 = 0.0;
 float humedad = 0.0;

//****
//DECLARACION DE VARIABLES PARA SINCRONIZACION DE TIEMPO NTP
//****

//VARIABLE AUXILIAR PARA ALMACENAR EL VALOR DE TIEMPO NTP
// auxiliar para saber cuando se mostro el reloj digital
 time_t prevDisplay = 0; 

//VARIABLE DE DESFASE DE TIEMPO NTP
//(NO DEBERIA SER NECESARIO MODIFICARLA)
//desfase de hora: el sistema siempre recibe y entrega hora UTC, y por lo tanto
// esta variable siempre se mantiene a 0. Para usar tiempo local, habrá que cambiar su valor
// Establece el valor al desfase en segundos con tu tiempo local.
 const  long timeZoneOffset = 0L; 

//****
//DELAY ENTRE LECTURAS (ms) PARA EL SENSOR DE HUMEDAD 
//****
 int delay_lectura = 20;

//****
//DECLARACIONES PARA EL BUS 1-WIRE (SENSORES DE TEMPERATURA)
//****
//Creamos una instancia del objeto onewire
 OneWire oneWire(ONE_WIRE_BUS);
//pasamos la referencia del objeto a la libreria de DallasTemperature
 DallasTemperature sensors(&oneWire);

//****
//DIRECCIONES SENSORES 1-WIRE
//****
//variables para almacenar la direccion de los sensores
 DeviceAddress sensorTemp1;
 DeviceAddress sensorTemp2;
//Establecemos la precision del sensor (9,10,11,12 bytes). Afecta al tiempo de respuesta
#define TEMPERATURE_PRECISION 12


//CONFIGURACIÓN DE ARDUINO
void setup() {

  //Inicialización del puerto serie a 9600 baudios
   Serial.begin(9600);
   Serial.println("Sensor de temperatura y humedad. Inicializando...");
   
   //Establecemos la referencia analógica en DEFAULT (5V)
   analogReference(DEFAULT);
  
   //1-WIRE
   //incializacion sensores onewire
   sensors.begin();
   //busqueda de Dispositivos  1-wire, por método del indice
   // localizamos los dispositivos en el bus
   Serial.print("Buscando dispositivos 1-Wire...");
   Serial.print("Encontrados ");
   Serial.print(sensors.getDeviceCount(), DEC);
   Serial.println(" dispositivos.");
   //mostramos el modo de alimentacion de los dispositivos
   Serial.print("Alimentacion parasita: "); 
   if (sensors.isParasitePowerMode()) Serial.println("ON");
   else Serial.println("OFF");

   //****
   //BUSQUEDA DE DIRECCIONES POR INDICE
   //****
   //busqueda de direcciones por indice (Necesaria para obtener las direcciones la primera vez)
   if (!sensors.getAddress(sensorTemp1, 0)) Serial.println("No se puede encontrar la direccion del dispositivo 0"); 
   if (!sensors.getAddress(sensorTemp2, 1)) Serial.println("No se puede encontrar la direccion del dispositivo 1"); 
   //mostramos las direcciones obtenidas
   Serial.print("Direccion sensor 1 (id 0): ");
   printAddress(sensorTemp1);
   Serial.println();
   Serial.print("Direccion sensor 2 (id 1): ");
   printAddress(sensorTemp2);
   Serial.println();
   
   //establecemos la precision de los sensores
   sensors.setResolution(sensorTemp1, TEMPERATURE_PRECISION);
   sensors.setResolution(sensorTemp2, TEMPERATURE_PRECISION);

   //mostramos la precision establecida en cada sensor
   Serial.print("Resolucion sensor 1: ");
   Serial.print(sensors.getResolution(sensorTemp1), DEC); 
   Serial.println();

   Serial.print("Resolucion sensor 2: ");
   Serial.print(sensors.getResolution(sensorTemp2), DEC); 
   Serial.println();
   
   //INICIALIZACION ETHERNET Y SERVIDOR
   Ethernet.begin(mac, ip, gw, subnet);
   
   //****
   //INICIALIZACION SERVIDOR WEB LOCAL
   servidor.begin();
   
   //SINCRONIZACION NTP
   //establece la sincronia de tiempos y la mantiene 
   //NO ES NECESARIO ACTUALIZAR LA SINCRONIZACION A MANO
   //*****
   //IMPORTANTE: SI AL ENCENDER EL ARDUINO NO SE LOGRA SINCRONIZAR, 
   //NO SE INICIA LA EJECUCION
   //*****
   setSyncProvider(getNtpTime);
   while(timeStatus()== timeNotSet)   
    ; // Espera hasta que se recibe sincronizacion

   //finalizamos la inicializacion
   Serial.print("Finalizada inicializacion. Direccion IP: ");
   for (int i = 0; i < 4; i++) {
      Serial.print(int(ip[i]));
      if (i<3)
      {
       Serial.print(".");
      }
    }
    Serial.println();
}

//PROGRAMA PRINCIPAL
void loop() {

  //PROCESAMOS CADA SEGUNDO
  if( now() != prevDisplay) //si la hora ha cambiado (ha pasado 1 segundo)
  {
    //actualizamos la variable de hora con el tiempo actual
    prevDisplay = now();
    //mostramos la hora por serie
    digitalClockDisplay();  
    //leemos la temperatura de los sensores,
    medida_temperatura();
    //leemos la humedad
    medida_humedad();
    //imprimo sensores por puerto serie
    muestraSerie();

    //escritura en servidor remoto cada 5 minutos
    //Se incluye aqui dentro (en proceso cada 1s) para que se haga 1 sola escritura por segundo
    //si la ponemos fuera, y el ciclo total tarda menos de 1s, haría varias escrituras en un solo ciclo
    //El registro se hace cuando minuto es 0 y segundo es 0 (hora en punto)
    //y tambien cuando minuto es multiplo de 5 (el modulo con 5 es cero) y segundo es 0
    //para hacer la escritura una sola vez, en el segundo indicado
    //
    if ( (minute()==0 && second()==0) || (minute()%5==0 && second()==0) )
    {
      //envio al servidor remoto
      envioweb(temperaturaT1, temperaturaT2, humedad);
    }

  }

   //****
   //MOSTRAR DATOS EN SERVIDOR WEB LOCAL
   //****
   //llamo al servidor web local para mostrar datos
   servidorweb();
   
  //esperamos 1ms antes de volver a empezar
   delay(1);

}

   //****
   //FUNCION DE SERVIDOR WEB LOCAL
   //****
   //funcion servidorweb, para mostrar resultados directamente por web en la IP del arduino
void servidorweb()
{

  //SERVIDOR WEB
  //El código es una adaptación directa del servidor Web incluido en los ejemplos de Arduino
  //solo se han traducido y ampliado los comentarios, y adaptado la salida para que muestre
  //los valores de temperatura y humedad calculados
  //El servidor responde en la IP indicada en el comienzo del programa, en la parte de
  //inicialización ethernet, por la variable ip[]
  
  //creamos una nueva instancia del servidor, y la ponemos como disponible
  //para que el cliente se pueda conectar
  Client cliente =  servidor.available();
  //Comprobamos si se crea el servidor correctamente
  if (cliente) {
    //variable auxiliar: una petición http de un cliente termina con una linea en blanco
    boolean current_line_is_blank = true;
    //mientras tengamos un cliente conectado
    while (cliente.connected()) {
      //si el cliente esta disponible
      if (cliente.available()) {
        // leemos la peticion del cliente
        char c = cliente.read();
        //si hemos llegado al final de la línea (hemos recibido un 
        //caracter de salto de linea) y la linea esta vacia, la peticion
        //http ha finalizado, por lo tanto ya podemos mandar la respuesta
        if (c == '\n' && current_line_is_blank) {
          // devolvemos una cabecera HTTP estandar
          cliente.println("HTTP/1.1 200 OK");
          cliente.println("Content-Type: text/html");
          cliente.println();
          
          //comienza el código que se va a ver en la página

          //mostramos la IP del sensor
            cliente.print("<h1>WeatherLAN: sensor de temperatura y humedad</h1><a href='http://");
              for (int i = 0; i < 4; i++) {
                cliente.print(int(server[i]));
                if (i<3)
                {
                 cliente.print(".");
                }
              }
            cliente.print("/weatherlan/index.php'");
            cliente.print(">Historico de datos</a><br />Sensor en IP: ");
              for (int i = 0; i < 4; i++) {
                cliente.print(int(ip[i]));
                if (i<3)
                {
                 cliente.print(".");
                }
              }
            cliente.println("<br />Fecha: ");
            //fecha y hora
            cliente.print(day());
            cliente.print("/");
            cliente.print(month());
            cliente.print("/");
            cliente.print(year());
            cliente.print(" ");
            cliente.print(hour());
            cliente.print(":");
            cliente.print(minute());
            cliente.print(":");
            cliente.print(second());
            // Salida de los valores de los sensores
            cliente.print("<br /><h2>Temperatura T1: ");
            cliente.print(temperaturaT1);
            cliente.print(" &ordm C</h2><br /> <h2>Temperatura T2: ");
            cliente.print(temperaturaT2);
            cliente.print(" &ordm C</h2><br /> <h2>Humedad: ");
            cliente.print(humedad);
            //la ultima escritura la hacemos con println para que salte de linea al finalizar
            cliente.println(" % HR</h2><br />");
          break;
        }
        //esta parte es para saber si ya ha terminado la petición del cliente
        if (c == '\n') {
          // estamos empezando una nueva linea
          current_line_is_blank = true;
        } else if (c != '\r') {
          // hemos recibido algún caracter en la línea actual.
          current_line_is_blank = false;
        }
      }
    }
    //cerramos el cliente
    cliente.stop();
    // esperamos para permitir al navegador recibir los datos
    delay(1);
    }
}


//funcion envioweb, para almacenar datos en servidor remoto
void envioweb(float t1, float t2, float h1)
{
 
 Client client(server, 80);
 Serial.println("Envio de datos al servidor web:");
  //inicializacion cliente web por ethernet

 //creamos una clase cliente para que conecte al servidor indicado en el puerto 80 
    if (client.connect()) {
      Serial.print("Conectado OK: ");
      //comenzamos la peticion http
      client.print("GET /weatherlan/index.php?reg=1&t1=");
      Serial.print("GET /weatherlan/index.php?reg=1&t1=");
      //ponemos las variables
      client.print(t1);
      Serial.print(t1);
      client.print("&t2=");
      Serial.print("&t2=");
      client.print(t2);
      Serial.print(t2);
      client.print("&h1=");
      Serial.print("&h1=");
      client.print(h1);
      Serial.print(h1);
      client.print("&hora=");
      Serial.print("&hora=");
      client.print(hour());
      Serial.print(hour());
      client.print("&minuto=");
      Serial.print("&minuto=");
      client.print(minute());
      Serial.print(minute());
      client.print("&dia=");
      Serial.print("&dia=");
      client.print(day());
      Serial.print(day());
      client.print("&mes=");
      Serial.print("&mes=");
      client.print(month());
      Serial.print(month());
      client.print("&anyo=");
      Serial.print("&anyo=");
      client.print(year());
      Serial.print(year());
      //terminamos indicando el protocolo HTTP y un salto de linea
      //client.println(" HTTP/1.1");
      //Serial.print(" HTTP/1.1");
      //linea en blanco para que termine la peticion
      client.println();
      Serial.println();
    }
    else
    {
      Serial.println("Error al conectar. Compruebe la conexion.");
    }
    
    //paramos el cliente
    client.stop();
    delay(1);
}

//FUNCIONES MOSTRAR HORA POR SERIAL
void digitalClockDisplay(){
  // digital clock display of the time
  Serial.print("Fecha: ");
  Serial.print(day());
  Serial.print("/");
  Serial.print(month());
  Serial.print("/");
  Serial.print(year());
  Serial.print(" HORA: ");
  Serial.print(hour());
  printDigits(minute());
  printDigits(second());
  Serial.println(); 
}

void printDigits(int digits){
  // utility function for digital clock display: prints preceding colon and leading 0
  Serial.print(":");
  if(digits < 10)
    Serial.print('0');
  Serial.print(digits);
}

//FUNCIONES NTP

//obtiene la hora del servidor NTP, devuelve como Unsigned Long, mediante UDP
unsigned long getNtpTime()
{
  sendNTPpacket(SNTP_server_IP);
  delay(1000);
  if ( UdpBytewise.available() ) {
    for(int i=0; i < 40; i++)
       UdpBytewise.read(); // ignore every field except the time
    const unsigned long seventy_years = 2208988800UL + timeZoneOffset;        
    return getUlong() -  seventy_years;      
  }
  return 0; // return 0 if unable to get the time
}

//envia paquete NTP al servidor para hacer la consulta por UDP
unsigned long sendNTPpacket(byte *address)
{
  UdpBytewise.begin(123);
  UdpBytewise.beginPacket(address, 123);
  UdpBytewise.write(B11100011);   // LI, Version, Mode
  UdpBytewise.write(0);    // Stratum
  UdpBytewise.write(6);  // Polling Interval
  UdpBytewise.write(0xEC); // Peer Clock Precision
  write_n(0, 8);    // Root Delay & Root Dispersion
  UdpBytewise.write(49); 
  UdpBytewise.write(0x4E);
  UdpBytewise.write(49);
  UdpBytewise.write(52);
  write_n(0, 32); //Reference and time stamps  
  UdpBytewise.endPacket();   
}

//lee un Unisgned Long para UDP
unsigned long getUlong()
{
    unsigned long ulong = (unsigned long)UdpBytewise.read() << 24;
    ulong |= (unsigned long)UdpBytewise.read() << 16;
    ulong |= (unsigned long)UdpBytewise.read() << 8;
    ulong |= (unsigned long)UdpBytewise.read();
    return ulong;
}

//escribe bytes para UDP
void write_n(int what, int how_many)
{
  for( int i = 0; i < how_many; i++ )
    UdpBytewise.write(what);
}

//medida del sensor de humedad
void medida_humedad()
{
  
  //LECTURA DE VALORES EN LAS ENTRADAS ANALOGICAS
  //la lectura del dato en crudo (entero, de 0 a 1023) se obtiene con analogRead(pin)
  //tomamos 5 muestras del sensor para mejorar la estabilidad de las lecturas, 
  //con una pequeña pausa entre cada medida para permitir al conversor
  //analógico-digital recuperarse entre cada medida
  for (int i=0; i<5; i++)
  {
    analogRead(puertoH1);
    delay(delay_lectura);
    vhumedad[i] = analogRead(puertoH1);
  }
  
  //CALCULO DE VALORES MEDIOS Y CONVERSION A mV
  //inicializo a cero el acumulador
    mhumedad = 0.0;
    
  //sumamos las 5 lecturas
    for (int i=0; i<5; i++)
    {
      mhumedad = mhumedad + float(vhumedad[i]); 
    }
    
  //Dividimos el valor sumado entre 5 para obtener la media.
  //y el resultado lo multiplicamos por 5000 (5V) y dividimos por 1024 para
  //convertirlo a mV
  mhumedad = ((mhumedad / 5.0)*5000.0)/1024.0;
  
  //CONVERSION A VALORES REALES, EN LAS UNIDADES CORRESPONDIENTES (%HR)
  //estas conversiones se obtienen de las ecuaciones indicadas en las hojas de características
  //del componente.
 
  //En la humedad, los valores de salida normales del circuito son 0,8V para el 0% a 3,9V para el 100%
  //Tomamos la medida en mV y le restamos 800mv (voltaje de salida a 0%HR)
  //y el resultado lo dividimos por 31 (dado que el rango de salida del sensor, 800mV a 3900mV,
  //supone 3100mV de amplitud, y queremos convertir a %, por lo tanto dividimos entre 100, resultando 31)
  humedad = (mhumedad-800.0)/31;
}

void medida_temperatura()
{
  //lectura de temperatura de los sensores 1-wire
  sensors.requestTemperatures(); // Envia por el bus 1-wire el comando para leer temperaturas
  
  //almacena los valores recibidos de los sensores en las variables
  temperaturaT1 = sensors.getTempC(sensorTemp1);
  temperaturaT2 = sensors.getTempC(sensorTemp2);
}

void muestraSerie()
{
   //IMPRESION DE LOS VALORES POR EL PUERTO SERIE
    Serial.print("Temp 1: ");
    Serial.print(temperaturaT1);
    Serial.print("C; \t Temp 2: ");
    Serial.print(temperaturaT2);
    Serial.print("C; \t Humedad: ");
    Serial.print(humedad);
    Serial.println("%HR");
}


//funciones 1-wire
//Muestra por serie la direccion de un dispositivo
void printAddress(DeviceAddress deviceAddress)
{
  for (uint8_t i = 0; i < 8; i++)
  {
    // zero pad the address if necessary
    if (deviceAddress[i] < 16) Serial.print("0");
    Serial.print(deviceAddress[i], HEX);
  }
}

